use serde::{Deserialize, Serialize};

use crate::clustering::LavinaClient;

pub mod paths {
    pub const JOIN: &'static str = "/cluster/rooms/join";
    pub const LEAVE: &'static str = "/cluster/rooms/leave";
    pub const ADD_MESSAGE: &'static str = "/cluster/rooms/add_message";
    pub const SET_TOPIC: &'static str = "/cluster/rooms/set_topic";
}

#[derive(Serialize, Deserialize, Debug)]
pub struct JoinRoomReq<'a> {
    pub room_id: &'a str,
    pub player_id: &'a str,
}

#[derive(Serialize, Deserialize, Debug)]
pub struct LeaveRoomReq<'a> {
    pub room_id: &'a str,
    pub player_id: &'a str,
}

#[derive(Serialize, Deserialize, Debug)]
pub struct SendMessageReq<'a> {
    pub room_id: &'a str,
    pub player_id: &'a str,
    pub message: &'a str,
    pub created_at: &'a str,
}

#[derive(Serialize, Deserialize, Debug)]
pub struct SetRoomTopicReq<'a> {
    pub room_id: &'a str,
    pub player_id: &'a str,
    pub topic: &'a str,
}

impl LavinaClient {
    #[tracing::instrument(skip(self, req), name = "LavinaClient::join_room")]
    pub async fn join_room(&self, node_id: u32, req: JoinRoomReq<'_>) -> anyhow::Result<()> {
        self.send_request(node_id, paths::JOIN, req).await
    }

    #[tracing::instrument(skip(self, req), name = "LavinaClient::leave_room")]
    pub async fn leave_room(&self, node_id: u32, req: LeaveRoomReq<'_>) -> anyhow::Result<()> {
        self.send_request(node_id, paths::LEAVE, req).await
    }

    #[tracing::instrument(skip(self, req), name = "LavinaClient::send_room_message")]
    pub async fn send_room_message(&self, node_id: u32, req: SendMessageReq<'_>) -> anyhow::Result<()> {
        self.send_request(node_id, paths::ADD_MESSAGE, req).await
    }

    #[tracing::instrument(skip(self, req), name = "LavinaClient::set_room_topic")]
    pub async fn set_room_topic(&self, node_id: u32, req: SetRoomTopicReq<'_>) -> anyhow::Result<()> {
        self.send_request(node_id, paths::SET_TOPIC, req).await
    }
}
